X-Git-Url: https://git.mdrn.pl/librarian.git/blobdiff_plain/6ec15224f3482ea18d339df7e2acc160d799be43..b416c7d130f17ac3191d2ef39492cd7c4f70bef7:/librarian/html.py?ds=sidebyside
diff --git a/librarian/html.py b/librarian/html.py
index c2e0dea..1c141f3 100644
--- a/librarian/html.py
+++ b/librarian/html.py
@@ -1,7 +1,7 @@
# -*- coding: utf-8 -*-
#
# This file is part of Librarian, licensed under GNU Affero GPLv3 or later.
-# Copyright © Fundacja Nowoczesna Polska. See NOTICE for more information.
+# Copyright © Fundacja Nowoczesna Polska. See NOTICE for more information.
#
import os
import cStringIO
@@ -11,16 +11,12 @@ import copy
from lxml import etree
from librarian.parser import WLDocument
from librarian import XHTMLNS, ParseError
+from librarian import functions
from lxml.etree import XMLSyntaxError, XSLTApplyError
-ENTITY_SUBSTITUTIONS = [
- (u'---', u'â'),
- (u'--', u'â'),
- (u'...', u'â¦'),
- (u',,', u'â'),
- (u'"', u'â'),
-]
+functions.reg_substitute_entities()
+functions.reg_person_name()
STYLESHEETS = {
'legacy': 'xslt/book2html.xslt',
@@ -31,21 +27,17 @@ STYLESHEETS = {
def get_stylesheet(name):
return os.path.join(os.path.dirname(__file__), STYLESHEETS[name])
-def substitute_entities(context, text):
- """XPath extension function converting all entites in passed text."""
- if isinstance(text, list):
- text = ''.join(text)
- for entity, substitutution in ENTITY_SUBSTITUTIONS:
- text = text.replace(entity, substitutution)
- return text
-
-# Register substitute_entities function with lxml
-ns = etree.FunctionNamespace('http://wolnelektury.pl/functions')
-ns['substitute_entities'] = substitute_entities
+def html_has_content(text):
+ return etree.ETXPath('//p|//{%(ns)s}p|//h1|//{%(ns)s}h1' % {'ns': str(XHTMLNS)})(text)
def transform(input, output_filename=None, is_file=True, \
- parse_dublincore=True, stylesheet='legacy', options={}):
- """Transforms file input_filename in XML to output_filename in XHTML."""
+ parse_dublincore=True, stylesheet='legacy', options={}, flags=None):
+ """Transforms file input_filename in XML to output_filename in XHTML.
+
+ If output_filename is None, returns an XML,
+ otherwise returns True if file has been written,False if it hasn't.
+ File won't be written if it has no content.
+ """
# Parse XSLT
try:
style_filename = get_stylesheet(stylesheet)
@@ -58,20 +50,29 @@ def transform(input, output_filename=None, is_file=True, \
document = WLDocument.from_string(input, True, \
parse_dublincore=parse_dublincore)
+ if flags:
+ for flag in flags:
+ document.edoc.getroot().set(flag, 'yes')
+
+ document.clean_ed_note()
+
result = document.transform(style, **options)
- del document # no longer needed large object :)
-
- if etree.ETXPath('//p|//{%s}p' % str(XHTMLNS))(result) is not None:
+ del document # no longer needed large object :)
+
+ if html_has_content(result):
add_anchors(result.getroot())
add_table_of_contents(result.getroot())
-
+
if output_filename is not None:
- result.write(output_filename, xml_declaration=False, pretty_print=True, encoding='utf-8')
+ result.write(output_filename, method='html', xml_declaration=False, pretty_print=True, encoding='utf-8')
else:
return result
return True
else:
- return ""
+ if output_filename is not None:
+ return False
+ else:
+ return ""
except KeyError:
raise ValueError("'%s' is not a valid stylesheet.")
except (XMLSyntaxError, XSLTApplyError), e:
@@ -126,7 +127,13 @@ def extract_fragments(input_filename):
open_fragments = {}
closed_fragments = {}
- for event, element in etree.iterparse(input_filename, events=('start', 'end')):
+ # iterparse would die on a HTML document
+ parser = etree.HTMLParser(encoding='utf-8')
+ buf = cStringIO.StringIO()
+ buf.write(etree.tostring(etree.parse(input_filename, parser).getroot()[0][0], encoding='utf-8'))
+ buf.seek(0)
+
+ for event, element in etree.iterparse(buf, events=('start', 'end')):
# Process begin and end elements
if element.get('class', '') in ('theme-begin', 'theme-end'):
if not event == 'end': continue # Process elements only once, on end event
@@ -188,7 +195,7 @@ def add_anchor(element, prefix, with_link=True, with_target=True, link_text=None
anchor.tail = element.text
element.text = u''
element.insert(0, anchor)
-
+
if with_target:
anchor_target = etree.Element('a', name='%s' % prefix)
anchor_target.set('class', 'target')
@@ -210,9 +217,10 @@ def add_anchors(root):
counter = 1
for element in root.iterdescendants():
if any_ancestor(element, lambda e: e.get('class') in ('note', 'motto', 'motto_podpis', 'dedication')
+ or e.get('id') == 'nota_red'
or e.tag == 'blockquote'):
continue
-
+
if element.tag == 'p' and 'verse' in element.get('class', ''):
if counter == 1 or counter % 5 == 0:
add_anchor(element, "f%d" % counter, link_text=counter)
@@ -229,14 +237,14 @@ def add_table_of_contents(root):
if element.tag in ('h2', 'h3'):
if any_ancestor(element, lambda e: e.get('id') in ('footnotes',) or e.get('class') in ('person-list',)):
continue
-
+
if element.tag == 'h3' and len(sections) and sections[-1][1] == 'h2':
sections[-1][3].append((counter, element.tag, ''.join(element.xpath('text()')), []))
else:
sections.append((counter, element.tag, ''.join(element.xpath('text()')), []))
add_anchor(element, "s%d" % counter, with_link=False)
counter += 1
-
+
toc = etree.Element('div')
toc.set('id', 'toc')
toc_header = etree.SubElement(toc, 'h2')
@@ -246,12 +254,24 @@ def add_table_of_contents(root):
for n, section, text, subsections in sections:
section_element = etree.SubElement(toc_list, 'li')
add_anchor(section_element, "s%d" % n, with_target=False, link_text=text)
-
+
if len(subsections):
subsection_list = etree.SubElement(section_element, 'ol')
for n, subsection, text, _ in subsections:
subsection_element = etree.SubElement(subsection_list, 'li')
add_anchor(subsection_element, "s%d" % n, with_target=False, link_text=text)
-
+
root.insert(0, toc)
+
+def extract_annotations(html_path):
+ """For each annotation, yields a tuple: anchor, text, html."""
+ parser = etree.HTMLParser(encoding='utf-8')
+ tree = etree.parse(html_path, parser)
+ for footnote in tree.find('//*[@id="footnotes"]').findall('div'):
+ anchor = footnote.find('a[@href]').get('href')
+ del footnote[:2]
+ text_str = etree.tostring(footnote, method='text', encoding='utf-8')
+ html_str = etree.tostring(footnote, method='html', encoding='utf-8')
+ yield anchor, text_str, html_str
+